package be.ikkeflikkeri.engine;

import java.nio.FloatBuffer;

import org.lwjgl.BufferUtils;

public class Matrix4f
{
	private float[][] matrix;
	
	public Matrix4f()
	{
		matrix = new float[4][4];
	}

	public Matrix4f initIdentity()
	{
		setMatrix
		(
			1, 0, 0, 0,
			0, 1, 0, 0,
			0, 0, 1, 0,
			0, 0, 0, 1
		);
		
		return this;
	}
	
	public Matrix4f initTranslation(float x, float y, float z)
	{
		setMatrix
		(
			1, 0, 0, x,
			0, 1, 0, y,
			0, 0, 1, z,
			0, 0, 0, 1
		);
		
		return this;
	}
	
	public Matrix4f initRotation(float x, float y, float z)
	{
		Matrix4f rx = new Matrix4f();
		Matrix4f ry = new Matrix4f();
		Matrix4f rz = new Matrix4f();
		
		x = (float)Math.toRadians(x);
		y = (float)Math.toRadians(y);
		z = (float)Math.toRadians(z);
		
		rx.setMatrix
		(
			1, 0, 0, 0,
			0, (float)Math.cos(x), -(float)Math.sin(x), 0,
			0, (float)Math.sin(x), (float)Math.cos(x), 0,
			0, 0, 0, 1
		);

		ry.setMatrix
		(
			(float)Math.cos(y), 0, -(float)Math.sin(y), 0,
			0, 1, 0, 0,
			(float)Math.sin(y), 0, (float)Math.cos(y), 0,
			0, 0, 0, 1
		);
		
		rz.setMatrix
		(
			(float)Math.cos(z), -(float)Math.sin(z), 0, 0,
			(float)Math.sin(z), (float)Math.cos(z), 0, 0,
			0, 0, 1, 0,
			0, 0, 0, 1
		);

		matrix = rz.multiply(ry.multiply(rx)).getMatrix();
		
		return this;
	}
	
	public Matrix4f initScale(float x, float y, float z)
	{
		setMatrix
		(
			x, 0, 0, 0,
			0, y, 0, 0,
			0, 0, z, 0,
			0, 0, 0, 1
		);

		return this;
	}
	
	public Matrix4f initProjection(float fov, float width, float height, float zNear, float zFar)
	{
		float ar = width/height;
		float tanHalfFOV = (float)Math.tan(Math.toRadians(fov / 2));
		float zRange = zNear - zFar;
		
		setMatrix
		(
				1.0f / (tanHalfFOV * ar), 0, 0, 0,
				0, 1.0f / tanHalfFOV, 0, 0,
				0, 0, (-zNear -zFar)/zRange, 2 * zFar * zNear / zRange,
				0, 0, 1, 0
		);

		return this;
	}
	
	public Matrix4f initCamera(Vector3f forward, Vector3f up)
	{
		Vector3f f = forward.normalized();
		Vector3f r = up.normalized().cross(f);
		Vector3f u = f.cross(r);
		
		setMatrix
		(
				r.getX(), r.getY(), r.getZ(), 0,
				u.getX(), u.getY(), u.getZ(), 0,
				f.getX(), f.getY(), f.getZ(), 0,
				0, 0, 0, 1
		);
		
		return this;
	}
	
	public Matrix4f multiply(Matrix4f r)
	{
		Matrix4f res = new Matrix4f();
		
		for(int i = 0; i < 4; i++)
		{
			for(int j = 0; j < 4; j++)
			{
				res.set(i, j, matrix[i][0] * r.get(0, j) + matrix[i][1] * r.get(1, j) + matrix[i][2] * r.get(2, j) + matrix[i][3] * r.get(3, j));
			}
		}
		
		return res;
	}
	
	public String toString()
	{
		return "[" + String.format("%.5f", get(0, 0)) + ", " + String.format("%.5f", get(0, 1)) + ", " + String.format("%.5f", get(0, 2)) + ", " + String.format("%.5f", get(0, 3)) + "]\n[" + String.format("%.5f", get(1, 0)) + ", " + String.format("%.5f", get(1, 1)) + ", " + String.format("%.5f", get(1, 2)) + ", " + String.format("%.5f", get(1, 3)) + "]\n[" + String.format("%.5f", get(2, 0)) + ", " + String.format("%.5f", get(2, 1)) + ", " + String.format("%.5f", get(2, 2)) + ", " + String.format("%.5f", get(2, 3)) + "]\n[" + String.format("%.5f", get(3, 0)) + ", " + String.format("%.5f", get(3, 1)) + ", " + String.format("%.5f", get(3, 2)) + ", " + String.format("%.5f", get(3, 3)) + "]";
	}
	
	public float[][] getMatrix()
	{
		float[][] res = new float[4][4];
		
		for(int i = 0; i < 4; i++)
			for(int j = 0; j < 4; j++)
				res[i][j] = matrix[i][j];
		
		return res;
	}
	
	public float get(int x, int y)
	{
		return matrix[x][y];
	}

	public void setMatrix(float m11, float m12, float m13, float m14, float m21, float m22, float m23, float m24, float m31, float m32, float m33, float m34, float m41, float m42, float m43, float m44)
	{
		set(0, 0, m11);
		set(0, 1, m12);
		set(0, 2, m13);
		set(0, 3, m14);
		set(1, 0, m21);
		set(1, 1, m22);
		set(1, 2, m23);
		set(1, 3, m24);
		set(2, 0, m31);
		set(2, 1, m32);
		set(2, 2, m33);
		set(2, 3, m34);
		set(3, 0, m41);
		set(3, 1, m42);
		set(3, 2, m43);
		set(3, 3, m44);
	}
	
	public void setMatrix(float[][] matrix)
	{
		this.matrix = matrix;
	}
	
	public void set(int x, int y, float value)
	{
		matrix[x][y] = value;
	}
	
	public FloatBuffer toFloatBuffer()
	{
		FloatBuffer buffer = BufferUtils.createFloatBuffer(16);
		
		for(int i = 0; i < 4; i++)
			for(int j = 0; j < 4; j++)
				buffer.put(get(i, j));
		
		buffer.flip();
		
		return buffer;
	}
}
