package be.ikkeflikkeri.engine;

import static org.lwjgl.opengl.GL11.*;
import static org.lwjgl.opengl.GL15.*;
import static org.lwjgl.opengl.GL20.*;

public class Mesh
{
	private int vbo;
	private int ibo;
	private int size;
	
	public Mesh()
	{
		vbo = glGenBuffers();
		ibo = glGenBuffers();
		size = 0;
	}
	
	public static Mesh load(String filename, boolean calcNormals)
	{
		return ResourceLoader.loadMesh(filename, calcNormals);
	}
	
	public void addVertices(VertexArray vertices, IndexArray indices)
	{
		addVertices(vertices, indices, false);
	}
	
	public void addVertices(VertexArray vertices, IndexArray indices, boolean calcNormals)
	{
		if(calcNormals)
		{
			calcNormals(vertices, indices);
		}
		
		size = indices.getSize() * Index.SIZE;
		
		glBindBuffer(GL_ARRAY_BUFFER, vbo);
		glBufferData(GL_ARRAY_BUFFER, vertices.toFloatBuffer(), GL_STATIC_DRAW);
		
		glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, ibo);
		glBufferData(GL_ELEMENT_ARRAY_BUFFER, indices.toFloatBuffer(), GL_STATIC_DRAW);
	}
	
	public void render()
	{
		glEnableVertexAttribArray(0);
		glEnableVertexAttribArray(1);
		glEnableVertexAttribArray(2);
		
		glBindBuffer(GL_ARRAY_BUFFER, vbo);
		glVertexAttribPointer(0, 3, GL_FLOAT, false, Vertex.SIZE * 4, 0);
		glVertexAttribPointer(1, 2, GL_FLOAT, false, Vertex.SIZE * 4, 12);
		glVertexAttribPointer(2, 3, GL_FLOAT, false, Vertex.SIZE * 4, 20);
		
		glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, ibo);
		glDrawElements(GL_TRIANGLES, size, GL_UNSIGNED_INT, 0);
		
		glDisableVertexAttribArray(0);
		glDisableVertexAttribArray(1);
		glDisableVertexAttribArray(2);
	}
	
	private void calcNormals(VertexArray vertices, IndexArray indices)
	{
		for(int i = 0; i < indices.getSize(); i++)
		{
			int i0 = indices.get(i).getI1();
			int i1 = indices.get(i).getI2();
			int i2 = indices.get(i).getI3();
			
			Vector3f v1 = vertices.get(i1).getPosition().substract(vertices.get(i0).getPosition());
			Vector3f v2 = vertices.get(i2).getPosition().substract(vertices.get(i0).getPosition());
			
			Vector3f normal = v1.cross(v2).normalized();
			
			vertices.get(i0).setNormal(vertices.get(i0).getNormal().add(normal));
			vertices.get(i1).setNormal(vertices.get(i1).getNormal().add(normal));
			vertices.get(i2).setNormal(vertices.get(i2).getNormal().add(normal));
		}
		
		for(int i = 0; i < vertices.getSize(); i++)
		{
			vertices.get(i).setNormal(vertices.get(i).getNormal().normalized());
		}
	}
}
