package be.ikkeflikkeri.engine;

public class PhongShader extends Shader
{
	private static PhongShader shader = new PhongShader();

	public static PhongShader getInstance()
	{
		return shader;
	}
	
	private static Vector3f ambientLight = new Vector3f(0.3f, 0.3f, 0.3f);
	private static DirectionalLight directionalLight = new DirectionalLight(new BaseLight(new Vector3f(1, 1, 1), 0), new Vector3f(0, 0, 0));
	
	public PhongShader()
	{
		super();
		
		addVertexShader(Shader.load("phong.vs"));
		addFragmentShader(Shader.load("phong.fs"));
		
		compile();
		
		addUniform("transform");
		addUniform("transformProjected");
		addUniform("baseColor");
		addUniform("ambientLight");
		addUniform("light.base.color");
		addUniform("light.base.intensity");
		addUniform("light.direction");
		addUniform("specularIntensity");
		addUniform("specularExponent");
		addUniform("cameraPosition");
	}
	
	public void updateUniforms(Matrix4f worldMatrix, Matrix4f projectedMatrix, Material material)
	{
		if(material.getTexture() != null)
			material.getTexture().bind();
		else
			Renderer.unbindTextures();
		
		setUniform("transform", worldMatrix);
		setUniform("transformProjected", projectedMatrix);
		setUniform("baseColor", material.getColor());
		setUniform("ambientLight", ambientLight);
		setUniform("light", directionalLight);
		setUniform("cameraPosition", Renderer.getCamera().getPosition());
		setUniformf("specularIntensity", material.getSpecularIntensity());
		setUniformf("specularExponent", material.getSpecularExponent());
	}

	public static Vector3f getAmbientLight()
	{
		return PhongShader.ambientLight;
	}

	public static void setAmbientLight(Vector3f ambientLight)
	{
		PhongShader.ambientLight = ambientLight;
	}
	
	public static DirectionalLight getDirectionalLight()
	{
		return directionalLight;
	}

	public static void setDirectionalLight(DirectionalLight directionalLight)
	{
		PhongShader.directionalLight = directionalLight;
	}

	public void setUniform(String uniformName, BaseLight light)
	{
		setUniform(uniformName + ".color", light.getColor());
		setUniformf(uniformName + ".intensity", light.getIntensity());
	}
	
	public void setUniform(String uniformName, DirectionalLight light)
	{
		setUniform(uniformName + ".base", light.getBase());
		setUniform(uniformName + ".direction", light.getDirection());
	}
}
