package be.ikkeflikkeri.engine;

import static org.lwjgl.opengl.GL11.*;
import static org.lwjgl.opengl.GL32.*;

public class Renderer
{
	private static boolean isWireframeMode = false;
	private static boolean isCullFace = true;
	
	public static void initialize()
	{
		if(!Window.isCreated())
		{
			Debug.error("Error: No OpenGL context, create a window before initializing the renderer!");
			Engine.forceStop();
		}
		
		glClearColor(0.0f, 0.0f, 0.0f, 0.0f);

		glFrontFace(GL_CW);
		glCullFace(GL_BACK);
		glEnable(GL_CULL_FACE);
		glEnable(GL_DEPTH_TEST);
		glEnable(GL_DEPTH_CLAMP);
		glEnable(GL_TEXTURE_2D);
 	}
	
	public static void clearScreen()
	{
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	}
	
	public static void setClearScreenColor(float x, float y, float z)
	{
		glClearColor(x, y, z, 1.0f);
	}
	
	public static void setClearScreenColor(Vector3f color)
	{
		glClearColor(color.getX(), color.getY(), color.getZ(), 1.0f);
	}
	
	public static void setClearScreenColor(Color color)
	{
		glClearColor(color.toVector3f().getX(), color.toVector3f().getY(), color.toVector3f().getZ(), 1.0f);
	}
	
	public static void setProjection(float fov, float zNear, float zFar)
	{
		Transform.setProjection(fov, zNear, zFar);
	}
	
	public static void setTextures(boolean enabled)
	{
		if(enabled)
			glEnable(GL_TEXTURE_2D);
		else
			glDisable(GL_TEXTURE_2D);
	}
	
	public static void setWireframeMode(boolean enabled)
	{
		isWireframeMode = enabled;
		
		if(isWireframeMode)
		{
			glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
		}
		else
		{
			glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
		}
	}
	public static boolean getWireframeMode()
	{
		return isWireframeMode;
	}
	
	public static void setCullFace(boolean enabled)
	{
		isCullFace = enabled;
		
		if(isCullFace)
		{
			glEnable(GL_CULL_FACE);
		}
		else
		{
			glDisable(GL_CULL_FACE);
		}
	}
	public static boolean getCullFace()
	{
		return isCullFace;
	}
	
	public static void setCamera(Camera camera)
	{
		Transform.setCamera(camera);
	}
	
	public static Camera getCamera()
	{
		return Transform.getCamera();
	}

	public static void unbindTextures()
	{
		glBindTexture(GL_TEXTURE_2D, 0);
	}
	
	public static String getOpenGLVersion()
	{
		return glGetString(GL_VERSION);
	}
}
