package be.ikkeflikkeri.engine;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;

import org.newdawn.slick.opengl.TextureLoader;

class ResourceLoader
{
	public static Texture loadTexture(String filename)
	{
		String extension = Utilities.getExtension(filename);

		try
		{
			int id = TextureLoader.getTexture(extension, new FileInputStream(new File("./res/textures/" + filename))).getTextureID();
			return new Texture(id);
		}
		catch (FileNotFoundException e)
		{
			Debug.error("Error: file not found: " + filename);
			Engine.forceStop();

		}
		catch (IOException e)
		{
			e.printStackTrace();
			Engine.forceStop();
		}
			
		return null;
	}
	public static String loadShader(String filename)
	{
		StringBuilder shaderSource = new StringBuilder();
		BufferedReader shaderReader = null;

		try
		{
			shaderReader = new BufferedReader(new FileReader("./res/shaders/" + filename));
			String line;

			while((line = shaderReader.readLine()) != null)
			{
				shaderSource.append(line).append("\n");
			}

			shaderReader.close();
		}
		catch(Exception e)
		{
			e.printStackTrace();
			Engine.forceStop();
		}


		return shaderSource.toString();
	}
	
	public static Mesh loadMesh(String filename, boolean calcNormals)
	{
		String extension = Utilities.getExtension(filename);
		
		if(!new File("./res/models/" + filename).isFile())
		{
			Debug.error("Error: file is not found for mesh data: " + filename);
			Engine.forceStop();
		}
		if(!extension.equals("obj"))
		{
			Debug.error("Error: file format not supported for mesh data: " + extension);
			Engine.forceStop();
		}
		
		VertexArray vertices = new VertexArray();
		IndexArray indices = new IndexArray();
		
		BufferedReader meshReader;
		
		try
		{
			meshReader = new BufferedReader(new FileReader("./res/models/" + filename));
		
			String line;
			while((line = meshReader.readLine()) != null)
			{
				String[] tokens = line.split(" ");
				tokens = Utilities.removeEmptyStrings(tokens);
				
				if(tokens.length == 0 || tokens[0].equals("#"))
					continue;
				else if(tokens[0].equals("v"))
				{
					vertices.add(new Vertex(new Vector3f(Float.valueOf(tokens[1]), Float.valueOf(tokens[2]), Float.valueOf(tokens[3]))));
				}
				else if(tokens[0].equals("f"))
				{
					indices.add(new Index(Integer.parseInt(tokens[1].split("/")[0]) - 1, Integer.parseInt(tokens[2].split("/")[0]) - 1, Integer.parseInt(tokens[3].split("/")[0]) - 1));
					
					if(tokens.length > 4)
					{
						indices.add(new Index(Integer.parseInt(tokens[1].split("/")[0]) - 1, Integer.parseInt(tokens[3].split("/")[0]) - 1, Integer.parseInt(tokens[4].split("/")[0]) - 1));
					}
				}
			}
			
			meshReader.close();
			
			Mesh mesh = new Mesh();
			mesh.addVertices(vertices, indices, calcNormals);
			
			return mesh;
		}
		catch (Exception e)
		{
			e.printStackTrace();
			Engine.forceStop();
		}
		
		return null;
	}
}
