package be.ikkeflikkeri.engine;

import static org.lwjgl.opengl.GL20.*;
import static org.lwjgl.opengl.GL32.*;

import java.util.HashMap;

public class Shader
{
	private int program;
	private HashMap<String, Integer> uniforms;

	public Shader()
	{
		program = glCreateProgram();
		uniforms = new HashMap<String, Integer>();

		if(program == 0)
		{
			Debug.error("Could not find valid memory location for the program!");
			Engine.forceStop();
		}
	}

	public void bind()
	{
		glUseProgram(program);
	}
	
	public void updateUniforms(Matrix4f worldMatrix, Matrix4f projectedMatrix, Material material)
	{
		
	}

	public static String load(String filename)
	{
		return ResourceLoader.loadShader(filename);
	}
	
	public void addUniform(String uniform)
	{
		int uniformLocation = glGetUniformLocation(program, uniform);

		if(uniformLocation == -1)
		{
			Debug.error("Error: Could not find uniform: " + uniform);
			Engine.forceStop();
		}

		uniforms.put(uniform, uniformLocation);
	}

	public void addVertexShader(String text)
	{
		addProgram(text, GL_VERTEX_SHADER);
	}

	public void addGeometryShader(String text)
	{
		addProgram(text, GL_GEOMETRY_SHADER);
	}

	public void addFragmentShader(String text)
	{
		addProgram(text, GL_FRAGMENT_SHADER);
	}

	public void compile()
	{
		glLinkProgram(program);

		if(glGetProgrami(program, GL_LINK_STATUS) == 0)
		{
			Debug.error(glGetProgramInfoLog(program, 1024));
			Engine.forceStop();
		}

		glValidateProgram(program);

		if(glGetProgrami(program, GL_VALIDATE_STATUS) == 0)
		{
			Debug.error(glGetProgramInfoLog(program, 1024));
			Engine.forceStop();
		}
	}

	private void addProgram(String text, int type)
	{
		int shader = glCreateShader(type);

		if(shader == 0)
		{
			Debug.error("Could not find valid memory location for the shader!");
			Engine.forceStop();
		}

		glShaderSource(shader, text);
		glCompileShader(shader);

		if(glGetShaderi(shader, GL_COMPILE_STATUS) == 0)
		{
			Debug.error(glGetShaderInfoLog(shader, 1024));
			Engine.forceStop();
		}

		glAttachShader(program, shader);
	}

	public int getUniform(String uniformName)
	{
		if(uniforms.get(uniformName) == null)
		{
			Debug.error("Error: uniform '" + uniformName + "' is not found!");
			Engine.forceStop();
		}
		
		return uniforms.get(uniformName);
	}
	
	public void setUniformi(String uniformName, int value)
	{
		glUniform1i(getUniform(uniformName), value);
	}

	public void setUniformf(String uniformName, float value)
	{
		glUniform1f(getUniform(uniformName), value);
	}

	public void setUniform(String uniformName, Vector3f value)
	{
		glUniform3f(getUniform(uniformName), value.getX(), value.getY(), value.getZ());
	}

	public void setUniform(String uniformName, Matrix4f value)
	{
		glUniformMatrix4(getUniform(uniformName), true, value.toFloatBuffer());
	}
}
