package be.ikkeflikkeri.engine;

public class Transform
{
	private static float zNear;
	private static float zFar;
	private static float fov;
	private static Camera camera;
	
	private Vector3f translation;
	private Vector3f rotation;
	private Vector3f scale;

	public Transform()
	{
		translation = new Vector3f(0, 0, 0);
		rotation = new Vector3f(0, 0, 0);
		scale = new Vector3f(1, 1, 1);
	}
	
	public Matrix4f getTransformation()
	{
		Matrix4f translationMatrix = new Matrix4f().initTranslation(translation.getX(), translation.getY(), translation.getZ());
		Matrix4f rotationMatrix = new Matrix4f().initRotation(rotation.getX(), rotation.getY(), rotation.getZ());
		Matrix4f scaleMatrix = new Matrix4f().initScale(scale.getX(), scale.getY(), scale.getZ());
		
		return translationMatrix.multiply(rotationMatrix.multiply(scaleMatrix));
	}
	
	public Matrix4f getProjectedTransformation()
	{
		Matrix4f transformationMatrix = getTransformation();
		Matrix4f projectionMatrix = new Matrix4f().initProjection(fov, Window.getWidth(), Window.getHeight(), zNear, zFar);
		Matrix4f cameraRotation = new Matrix4f().initCamera(camera.getForward(), camera.getUp());
		Matrix4f cameraTranslation = new Matrix4f().initTranslation(-camera.getPosition().getX(), -camera.getPosition().getY(), -camera.getPosition().getZ());
		
		return projectionMatrix.multiply(cameraRotation.multiply(cameraTranslation.multiply(transformationMatrix)));
	}
	
	public Vector3f getTranslation()
	{
		return translation;
	}
	
	public void setTranslation(Vector3f translation)
	{
		this.translation = translation;
	}
	
	public void setTranslation(float x, float y, float z)
	{
		this.translation = new Vector3f(x, y, z);
	}

	public Vector3f getRotation()
	{
		return rotation;
	}

	public void setRotation(Vector3f rotation)
	{
		this.rotation = rotation;
	}
	
	public void setRotation(float x, float y, float z)
	{
		this.rotation = new Vector3f(x, y, z);
	}

	public Vector3f getScale()
	{
		return scale;
	}

	public void setScale(Vector3f scale)
	{
		this.scale = scale;
	}
	
	public void setScale(float x, float y, float z)
	{
		this.scale = new Vector3f(x, y, z);
	}
	
	static void setProjection(float fov, float zNear, float zFar)
	{
		Transform.fov = fov;
		Transform.zNear = zNear;
		Transform.zFar = zFar;
	}
	
	public static float getFOV()
	{
		return fov;
	}
	
	public static float getZNear()
	{
		return zNear;
	}
	
	public static float getZFar()
	{
		return zFar;
	}

	static Camera getCamera()
	{
		return camera;
	}

	static void setCamera(Camera camera)
	{
		Transform.camera = camera;
	}
}
