package be.ikkeflikkeri.game;

import be.ikkeflikkeri.engine.Camera;
import be.ikkeflikkeri.engine.Color;
import be.ikkeflikkeri.engine.DirectionalLight;
import be.ikkeflikkeri.engine.Engine;
import be.ikkeflikkeri.engine.Index;
import be.ikkeflikkeri.engine.IndexArray;
import be.ikkeflikkeri.engine.Input;
import be.ikkeflikkeri.engine.Keys;
import be.ikkeflikkeri.engine.Material;
import be.ikkeflikkeri.engine.Mesh;
import be.ikkeflikkeri.engine.PhongShader;
import be.ikkeflikkeri.engine.Renderer;
import be.ikkeflikkeri.engine.Shader;
import be.ikkeflikkeri.engine.Texture;
import be.ikkeflikkeri.engine.Time;
import be.ikkeflikkeri.engine.Transform;
import be.ikkeflikkeri.engine.Vector2f;
import be.ikkeflikkeri.engine.Vector3f;
import be.ikkeflikkeri.engine.Vertex;
import be.ikkeflikkeri.engine.VertexArray;
import be.ikkeflikkeri.engine.Window;

public class Game
{
	private Mesh mesh;
	private Mesh mesh1;
	private Mesh mesh2;
	private Shader shader;
	private Material material;
	private Material material1;
	private Material material2;
	private Transform transform;
	private Camera camera;
	
	public void initialize()
	{
		Window.setTitle("Game Engine - by ikkeflikkeri");
		Window.setResolution(640, 480, false);
		Window.setAntiAliasing(8);
		Window.setVSync(false);
		Window.setFPS(5000);
		Window.create();

		VertexArray vertices1 = new VertexArray();
		vertices1.add(new Vertex(new Vector3f(1, -1, -1), new Vector2f(0, 1)));
		vertices1.add(new Vertex(new Vector3f(1, -1, 1), new Vector2f(1, 1)));
		vertices1.add(new Vertex(new Vector3f(-1, -1, 1), new Vector2f(0, 1)));
		vertices1.add(new Vertex(new Vector3f(-1, -1, -1), new Vector2f(1, 1)));
		vertices1.add(new Vertex(new Vector3f(1, 1, -1), new Vector2f(0, 0)));
		vertices1.add(new Vertex(new Vector3f(1, 1, 1), new Vector2f(1, 0)));
		vertices1.add(new Vertex(new Vector3f(-1, 1, 1), new Vector2f(0, 0)));
		vertices1.add(new Vertex(new Vector3f(-1, 1, -1), new Vector2f(1, 0)));

		IndexArray indices1 = new IndexArray();
		indices1.add(new Index(0, 1, 2));
		indices1.add(new Index(4, 7, 6));
		indices1.add(new Index(0, 4, 5));
		indices1.add(new Index(1, 5, 2)); 
		indices1.add(new Index(2, 6, 3));
		indices1.add(new Index(4, 0, 3));
		indices1.add(new Index(3, 0, 2));
		indices1.add(new Index(5, 4, 6));
		indices1.add(new Index(1, 0, 5));
		indices1.add(new Index(5, 6, 2));
		indices1.add(new Index(6, 7, 3));
		indices1.add(new Index(7, 4, 3));

		VertexArray vertices2 = new VertexArray();
		vertices2.add(new Vertex(new Vector3f(-1, -1, 0.5773f), new Vector2f(0, 0)));
		vertices2.add(new Vertex(new Vector3f(0, -1, -1.15475f), new Vector2f(0.5f, 0)));
		vertices2.add(new Vertex(new Vector3f(1, -1, 0.5773f), new Vector2f(1, 0)));
		vertices2.add(new Vertex(new Vector3f(0, 1, -0), new Vector2f(0.5f, 1)));

		IndexArray indices2 = new IndexArray();
		indices2.add(new Index(0, 3, 1));
		indices2.add(new Index(1, 3, 2));
		indices2.add(new Index(2, 3, 0));
		indices2.add(new Index(1, 2, 0)); 
		
		camera = new Camera();
		
		mesh1 = new Mesh();
		mesh1.addVertices(vertices1, indices1, true);
		mesh2 = new Mesh();
		mesh2.addVertices(vertices2, indices2, true);
		mesh = mesh1;
		
		material1 = new Material(Texture.load("grass.jpg"), Color.WHITE.toVector3f(), 1, 16);
		material2 = new Material(Texture.load("texture.jpg"), Color.WHITE.toVector3f(), 2, 32);
		material = material1;
		
		transform = new Transform();
		shader = PhongShader.getInstance();
		
		PhongShader.setAmbientLight(new Vector3f(0.2f, 0.2f, 0.2f));
		PhongShader.setDirectionalLight(new DirectionalLight(Color.WHITE.toVector3f(), 0.8f, new Vector3f(1, 1, 1)));
		
		Renderer.initialize();
		Renderer.setWireframeMode(false);
		Renderer.setCullFace(true);
		Renderer.setClearScreenColor(0, 0, 0);
		Renderer.setProjection(70, 0.1f, 100.0f);
		Renderer.setCamera(camera);
	}

	public void input()
	{
		if (Window.isCloseRequested() || (Input.getKey(Keys.KEY_LALT) && Input.getKeyDown(Keys.KEY_F4)))
			Engine.stop();

		if (Input.getKeyDown(Keys.KEY_F1))
			Renderer.setWireframeMode(!Renderer.getWireframeMode());
		if (Input.getKeyDown(Keys.KEY_F2))
			Renderer.setCullFace(!Renderer.getCullFace());

		if(Input.getKeyDown(Keys.KEY_NUMPAD1))
			material = material1;
		if(Input.getKeyDown(Keys.KEY_NUMPAD2))
			material = material2;
		if(Input.getKeyDown(Keys.KEY_NUMPAD4))
			mesh = mesh1;
		if(Input.getKeyDown(Keys.KEY_NUMPAD5))
			mesh = mesh2;
		
		if (Input.getKey(Keys.KEY_Z))
			camera.move(camera.getForward(), 5 * (float)Time.getDelta());
		if (Input.getKey(Keys.KEY_S))
			camera.move(camera.getForward(), -5 * (float)Time.getDelta());
		if (Input.getKey(Keys.KEY_D))
			camera.move(camera.getLeft(), 5 * (float)Time.getDelta());
		if (Input.getKey(Keys.KEY_Q))
			camera.move(camera.getRight(), 5 * (float)Time.getDelta());
		if (Input.getKey(Keys.KEY_LSHIFT))
			camera.move(camera.getGlobalUp(), -5 * (float)Time.getDelta());
		if (Input.getKey(Keys.KEY_SPACE))
			camera.move(camera.getGlobalUp(), 5 * (float)Time.getDelta());
		
		if(Input.getMouseDown(Keys.MOUSE_LEFT))
		{
			Input.setMousePosition(Window.getCenter());
			Input.setCursorGrabbed(true);
		}
		if(Input.getKeyDown(Keys.KEY_ESCAPE))
		{
			Input.setCursorGrabbed(false);
		}
		
		if(Input.getCursorGrabbed())
		{
			Vector2f deltaPos = Input.getMousePosition().substract(Window.getCenter());

			if(deltaPos.getX() != 0)
				camera.rotateY(deltaPos.getX() * 0.2f);
			if(deltaPos.getY() != 0)
				camera.rotateX(-deltaPos.getY() * 0.2f);

			Input.setMousePosition(Window.getCenter());
		}
	}

	float temp = 0.0f;
	
	public void update()
	{
		Input.update();

		temp += Time.getDelta();
		
		transform.setTranslation(0, 0, 3);
		transform.setRotation(0, (float)Math.sin(temp) * 180, 0);
	}

	public void render()
	{
		Renderer.clearScreen();
		
		shader.bind();
		shader.updateUniforms(transform.getTransformation(), transform.getProjectedTransformation(), material);
		mesh.render();

		Window.render();
	}

	public void destoy()
	{
		Window.destroy();
	}
}
